<?php
/**
 * UploadBehavior class file.
 * @author Ahamed Rifaideen <rifajas@gmail.com>
 * @copyright Copyright &copy; Ahamed Rifaideen 2014
 * @license http://www.opensource.org/licenses/bsd-license.php New BSD License
 */

/**
 * Upload Behavior manages file upload and other file related operations such as delete file when record is deleted,
 * creates unique file name when uploading files, delete previous file when new file is uploaded for existing file 
 * and this behavior can be customized through its public properties.
 * 
 * public function behaviors() {
 *           
     * return array(
         * 'upload' => array(
             * 'class' => 'UploadBehavior',
             *  //default values
             * 'field' => 'file',
             * 'path' => 'assets',
             * 'renameIfExists' => true,
             * 'delete_previous_file' => true
             * 'delete_together' => true
         * )
     * );
 * 
 * }
 * 
 */
class UploadBehavior extends CActiveRecordBehavior {

    /**
     * name of the file field that this behavior manages
     */
    public $field = 'file';    
    /**
     * name of the path where uploaded files are saved
     * Note: path must not end with slash ( / )
     */
    public $path = 'assets';
    /**
     * rename the file when the file already exists
     */
    public $renameIfExists = true;
    /**
     * delete previous file when new file is uploaded
     */
    public $delete_previous_file = true;
    /**
     * delete file together with record
     */
    public $delete_together = true;
    
    /**
     * uploaded file extension
     */
    private $_extension;
    /**
     * error flag, default implementation will show error when this flag is true
     */
    private $hasError;
    /**
     * previous file name
     */
    private $previous_file;

    /**
     * assign file field to previous file
     * @param CModelEvent $event
     */
    public function afterFind($event) {
        $this->previous_file = $this->owner->{$this->field};
    }
    
    /**
     * deletes the file after the record is deleted
     * it deletes only when delete_together is set to true
     * @param CModelEvent $event
     */
    public function afterDelete($event) {
        
        if ( $this->delete_together === true ) {
            $f = $this->path . '/' . $this->owner->{$this->field};

            if ( file_exists($f) ) {
                @unlink($f);
            }
        }
    }

    /**
     * check for file upload and save the file when uploaded
     */
    public function beforeSave($event) {
        if ($this->owner->{$this->field} instanceof CUploadedFile) {

            $file = $this->owner->{$this->field}->name;
            $this->getUniqueFileName($file);

            if ($this->hasError === true) {
                $this->owner->addError($this->field, $file . ' already exists');
                $event->isValid = false;
                return false;
            }

            $this->owner->{$this->field}->saveAs($this->path . '/' . $file);
            $this->owner->{$this->field} = $file;

            if ($this->delete_previous_file === true) {
                $f = $this->path . '/' . $this->previous_file;

                if (file_exists($f) && $f != $file) {
                    @unlink($f);
                }
            }
        }
    }

    /**
     * load file instance before validates
     */
    public function beforeValidate($event) {
        $this->loadFile();
        if ($this->owner->{$this->field} === null) {
            $this->owner->{$this->field} = $this->previous_file;
        }
    }

    /**
     * load file instance
     */
    public function loadFile() {
        $this->owner->{$this->field} = CUploadedFile::getInstance($this->owner, $this->field);
    }

    /**
     * get unique file name.
     * @param string $file
     * @return boolean
     */
    public function getUniqueFileName(&$file) {
        if (file_exists($this->path . '/' . $file)) {
            if ($this->renameIfExists === true) {
                $this->_extension = '.' . CFileHelper::getExtension($file);
                $file = md5(microtime() . date('mdyHi')) . $this->_extension;
                $this->getUniqueFileName($file);
            } else {
                $this->hasError = true;
                return false;
            }
        } else {
            $this->hasError = false;
            return true;
        }
    }

}
